/* -*- Mode: C++; c-basic-offset:4 ; -*- */
/*  $Id: cxxtest.cc,v 1.5 2005/05/13 17:39:51 gropp Exp $
 *
 *  (C) 2001 by Argonne National Laboratory.
 *      See COPYRIGHT in top-level directory.
 */

#include "mpi2c++_test.h"

#ifdef USE_XMLOUT
#ifdef HAVE_IOSTREAM
// Prefer the newer version
#include <fstream>
#else
#include <fstream.h>
#endif
#endif

//
// Global variables
//

int comm_size = -1;
int my_rank = -1;
int to = -1;
int from = -1;
bool CANCEL_WORKS = false;
bool TIGHTLY_COUPLED = false;
const int version[2] = {1, 5};
const double Epsilon = 0.001;

//
// Local functions
//

static void check_minimals(void);


int main(int argc, char *argv[])
{

  // Start up MPI

  initialized1();

  MPI::Init();


  // Define some globals

  comm_size = MPI::COMM_WORLD.Get_size();
  my_rank   = MPI::COMM_WORLD.Get_rank();
  to        = (my_rank + 1) % comm_size;
  from      = (my_rank + comm_size - 1) % comm_size;

  // Announce

#ifdef VERBOSE
  // But only if we are being verbose
  if (my_rank == 0)
    cout << endl
	 << "Since we made it this far, we will assume that" << endl
	 << "MPI::Init() worked properly." << endl 
	 << "----------------------------------------------" 
	 << endl 
	 << "MPI-2 C++ bindings test suite" << endl 
	 << "------------------------------" << endl 
	 << "LSC Version " 
	 << version[0] << "." << version[1] << endl 
	 << endl 
	 << "*** There are delays built into some of the tests" << endl 
	 << "*** Please let them complete" << endl 
	 << "*** No test should take more than 10 seconds" << endl << endl;
#endif

  // Catch all fatal signals

  signal_init();

  // Check for minimal testing conditions in MPI environment

  check_minimals();

#if !defined(TEST_NAME) || !defined(TEST_ROUTINE)
#error "Must define with a test name"
#endif
Testing(TEST_NAME);
TEST_ROUTINE ();
Pass();

  if (my_rank == 0)
    cout << endl << "* MPI::Finalize..." << endl;

  MPI::COMM_WORLD.Barrier();

  MPI::Finalize();

#ifdef VERBOSE
  // Announce only if verbose.  Also not that the "All tests passed"
  // is based only on reaching this point; non-fatal errors could still
  // cause the summary to say "All tests passed"
  if (my_rank == 0)
    cout << endl << endl
	 << "Since we made it this far, we will assume that" << endl
	 << "MPI::Finalize() did what we wanted it to." << endl
	 << "(Or, at the very least, it didn't fail.)" << endl
	 << "-----------------------------------------------------------" << endl
	 << "MPI-2 C++ bindings test suite: All done.  All tests passed." << endl
	 << endl;
#endif
#ifdef USE_XMLOUT
  if (my_rank == 0) {
      ofstream xmlout;
      // Open for appending
      xmlout.open( "summary.xml", ios::app );
      // Currently, if we get to this point, the tests have passed (more or less)
      xmlout << "<MPITEST>" << endl
	     << "<NAME>" << TEST_NAME << "</NAME>" << endl
	     << "<NP>" << comm_size << "</NP>" << endl 
	     << "<STATUS>" << "pass" << "</STATUS>" << endl
	     << "</MPITEST>" << endl;
      // After status, could add <TESTDIFF> ... </TESTDIFF>
      xmlout.close();
  }
#endif

  return 0;
}


//
// Do meaningless work to burn up time
//
void
do_work(int top)
{
  double start1 = MPI::Wtime() + .25;
  top++;

  while(MPI::Wtime() < start1)
    continue;
}


//
// Check for minimal MPI environment
//
static void
check_minimals()
{
  if (my_rank == 0)
    cout << "Test suite running on " << comm_size << " nodes" << endl;

  if (comm_size < 2) {
    if (my_rank == 0) {
      cout << "Sorry, the MPI2C++ test suite must be run with at least 2 processors" << endl;
      cout << "Please re-run the program with 2 or more processors." << 
	endl << endl;
    }
    MPI::Finalize();
    exit(1);
  }
  
  if ((comm_size % 2) != 0) {
    if (my_rank == 0)
      cout << "The MPI2C++ test suite can only run on an even number" << endl 
	   << "of processors.  Please re-run the program with an even" << endl 
	   << "number of ranks." << endl << endl;

    MPI::Finalize();
    exit(1);
  }
}
